//  FXGLUT
#include "fxglut.h"
#include <fx.h>
#include <fx3d.h>
#include <fxkeys.h>
#include <list>

class FXGLUTWindow : public FXMainWindow
{
  FXDECLARE(FXGLUTWindow)
protected:
  FXGLCanvas *canvas;
  FXGLVisual *visual;
  int         glutID;
  int         activeMods;
  static int defaultDisplayMode;

  // CALLBACK POINTERS
  void (*displayFunc)(void);
  void (*reshapeFunc)(int width, int height);
  void (*keyboardFunc)(unsigned char key, int x, int y);
  void (*mouseFunc)(int button, int state, int x, int y);
  void (*motionFunc)(int x, int y);
  void (*passiveMotionFunc)(int x, int y);
  void (*idleFunc)(void);
  void (*specialFunc)(int key, int x, int y);
  void (*keyboardUpFunc)(unsigned char key, int x, int y);
  void (*specialUpFunc)(int key, int x, int y);

  friend class FXGLUT;

  void setId(int id) {
    glutID = id;
  }
  FXGLVisual* makeVisual();
  int mapSpecialKey(int key);
public:
  //  EVENT HANDLING FUNCTIONS
  long onPaint    (FXObject*,FXSelector,void*);
  long onConfigure(FXObject*,FXSelector,void*);
  long onChore    (FXObject*,FXSelector,void*);
  long onKeyPress (FXObject*,FXSelector,void*);
  long onKeyRelease(FXObject*,FXSelector,void*);
  long onMouse    (FXObject*,FXSelector,void*);
  long onMotion   (FXObject*,FXSelector,void*);
  long onOther    (FXObject*,FXSelector,void*);

public:
  enum {
    ID_CANVAS = FXMainWindow::ID_LAST,
    ID_CHORE
  };
  void create();
  int getId() const {
    return glutID;
  }
public:
  FXGLUTWindow() {}
  FXGLUTWindow(FXApp *a,const FXString &title, int x, int y, int w, int h);
  ~FXGLUTWindow();

};
int FXGLUTWindow::defaultDisplayMode = 0;

FXDEFMAP(FXGLUTWindow) FXGLUTWindowMap[]={
  FXMAPFUNC(SEL_PAINT,              FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onPaint),
  FXMAPFUNC(SEL_PAINT,              0,                       FXGLUTWindow::onPaint),
  FXMAPFUNC(SEL_CONFIGURE,          FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onConfigure),
  FXMAPFUNC(SEL_KEYPRESS,           FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE,         FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onKeyRelease),
  FXMAPFUNC(SEL_KEYPRESS,           0,                       FXGLUTWindow::onKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE,         0,                       FXGLUTWindow::onKeyRelease),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS,    FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onMouse),
  FXMAPFUNC(SEL_LEFTBUTTONRELEASE,  FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onMouse),
  FXMAPFUNC(SEL_MIDDLEBUTTONPRESS,  FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onMouse),
  FXMAPFUNC(SEL_MIDDLEBUTTONRELEASE,FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onMouse),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS,   FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onMouse),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onMouse),
  FXMAPFUNC(SEL_MOTION,             FXGLUTWindow::ID_CANVAS, FXGLUTWindow::onMotion),
  FXMAPFUNC(SEL_MOTION,             0,                       FXGLUTWindow::onMotion),

  FXMAPFUNC(SEL_CHORE,     FXGLUTWindow::ID_CHORE,    FXGLUTWindow::onChore),

  };

FXIMPLEMENT(FXGLUTWindow,FXMainWindow,FXGLUTWindowMap,ARRAYNUMBER(FXGLUTWindowMap))


FXGLUTWindow::FXGLUTWindow(FXApp*a,const FXString &title, int x, int y, int w, int h)
: FXMainWindow(a,title,NULL,NULL,
               DECOR_TITLE|DECOR_MINIMIZE|DECOR_MAXIMIZE|DECOR_CLOSE|
               DECOR_BORDER|DECOR_STRETCHABLE|DECOR_MENU,
               x,y,w,h),
  displayFunc(0),
  reshapeFunc(0),
  keyboardFunc(0),
  mouseFunc(0),
  motionFunc(0),
  passiveMotionFunc(0),
  idleFunc(0),
  specialFunc(0),
  keyboardUpFunc(0),
  specialUpFunc(0)
{
  visual=makeVisual();
  canvas=new FXGLCanvas(this,visual,this,ID_CANVAS,LAYOUT_FILL_X|LAYOUT_FILL_Y);
}
FXGLUTWindow::~FXGLUTWindow()
{
  if (idleFunc) getApp()->removeChore(getApp(),ID_CHORE);
}
void FXGLUTWindow::create(){
  FXMainWindow::create();
  show();
}
FXGLVisual* FXGLUTWindow::makeVisual()
{
  // RGB and RGBA are both 0.  So 0 is RGB/A  no difference.
  int visflags = 0;
  int mode = defaultDisplayMode;
  if (mode & FXGLUT_DOUBLE)
    visflags |= VISUAL_DOUBLEBUFFER;
  if (mode & FXGLUT_STEREO)
    visflags |= VISUAL_STEREO;
  if (mode & FXGLUT_INDEX)
    visflags |= VISUAL_INDEXCOLOR;
  if (mode & FXGLUT_LUMINANCE)
    visflags |= VISUAL_GRAYSCALE;
  FXGLVisual *vis = new FXGLVisual( getApp(), visflags );
  if (mode & FXGLUT_ACCUM) {
    vis->setAccumRedSize(8);
    vis->setAccumGreenSize(8);
    vis->setAccumBlueSize(8);
    vis->setAccumAlphaSize(8);
  }
  if (mode & FXGLUT_ALPHA) {
    vis->setAlphaSize(8);
  }
  if (mode & FXGLUT_STENCIL) {
    vis->setStencilSize(8);
  }
  if (mode & FXGLUT_DEPTH) {
    vis->setDepthSize(24);
  }
  return vis;
}
int FXGLUTWindow::mapSpecialKey(int key)
{
  switch(key) {
  case KEY_F1:   return FXGLUT_KEY_F1;
  case KEY_F2: return FXGLUT_KEY_F2;
  case KEY_F3: return FXGLUT_KEY_F3;
  case KEY_F4: return FXGLUT_KEY_F4;
  case KEY_F5: return FXGLUT_KEY_F5;
  case KEY_F6: return FXGLUT_KEY_F6;
  case KEY_F7: return FXGLUT_KEY_F7;
  case KEY_F8: return FXGLUT_KEY_F8;
  case KEY_F9: return FXGLUT_KEY_F9;
  case KEY_F10: return FXGLUT_KEY_F10;
  case KEY_F11: return FXGLUT_KEY_F11;
  case KEY_F12: return FXGLUT_KEY_F12;
  case KEY_KP_Left:
  case KEY_Left: return FXGLUT_KEY_LEFT;
  case KEY_KP_Up:
  case KEY_Up: return FXGLUT_KEY_UP;
  case KEY_KP_Right:
  case KEY_Right: return FXGLUT_KEY_RIGHT;
  case KEY_KP_Down:
  case KEY_Down: return FXGLUT_KEY_DOWN;
  case KEY_KP_Page_Up:
  case KEY_Page_Up: return FXGLUT_KEY_PAGE_UP;
  case KEY_KP_Page_Down:
  case KEY_Page_Down: return FXGLUT_KEY_PAGE_DOWN;
  case KEY_KP_Home:
  case KEY_Home: return FXGLUT_KEY_HOME;
  case KEY_KP_End: 
  case KEY_End: return FXGLUT_KEY_END;
  case KEY_KP_Insert: 
  case KEY_Insert: return FXGLUT_KEY_INSERT;
  }
  return 0;
}

long FXGLUTWindow::onPaint(FXObject*,FXSelector,void*)
{
  if (displayFunc && canvas->makeCurrent()) {
    displayFunc();
  }
  return 1;
}
long FXGLUTWindow::onConfigure(FXObject*,FXSelector,void*)
{
  if(reshapeFunc && canvas->makeCurrent()){
    reshapeFunc(canvas->getWidth(),canvas->getHeight());
  }
  return 1;
}
long FXGLUTWindow::onChore(FXObject*,FXSelector,void*)
{
  if (idleFunc && canvas->makeCurrent()) {
    idleFunc();
    getApp()->addChore(this,ID_CHORE);
  }
  return 1;
}
long FXGLUTWindow::onKeyPress(FXObject*sender,FXSelector sel,void*ptr)
{
  if (!keyboardFunc && !specialFunc) return 1;
  FXEvent *e = (FXEvent*)ptr;
  activeMods = e->state;
  if (e->text.length()!=0) {
    // Handle regular printable character key
    if (!keyboardFunc || !canvas->makeCurrent()) return 1;
    keyboardFunc(e->text[0], e->win_x, e->win_y);
  }
  else {
    if (!specialFunc || !canvas->makeCurrent()) return 1;
    int glutkey = mapSpecialKey(e->code);
    if (glutkey>=0) 
      specialFunc(glutkey, e->win_x, e->win_y);
  }
  return 1;
}
long FXGLUTWindow::onKeyRelease(FXObject*sender,FXSelector sel,void*ptr)
{
  if (!keyboardUpFunc && !specialUpFunc) return 1;
  FXEvent *e = (FXEvent*)ptr;
  activeMods = e->state;
  if (e->text.length()!=0) {
    // Handle regular printable character key
    if (!keyboardUpFunc) return 1;
    if (!canvas->makeCurrent()) return 1;
    keyboardUpFunc(e->text[0], e->win_x, e->win_y);
  }
  else {
    if (!specialUpFunc) return 1;
    if (!canvas->makeCurrent()) return 1;
    int glutkey = mapSpecialKey(e->code);
    if (glutkey>=0) 
      specialUpFunc(glutkey, e->win_x, e->win_y);
  }
  return 1;
}
long FXGLUTWindow::onMouse(FXObject*sender,FXSelector sel,void*ptr)
{
  if (!mouseFunc || !canvas->makeCurrent()) return 1;

  FXEvent *e = (FXEvent*)ptr;
  activeMods = e->state;
  int button;
  int state;
  switch(e->code) {
  case LEFTBUTTON:
    button = FXGLUT_LEFT_BUTTON;
    state = (e->state&LEFTBUTTONMASK)?FXGLUT_DOWN:FXGLUT_UP;
    break;
  case MIDDLEBUTTON:
    button = FXGLUT_MIDDLE_BUTTON;
    state = (e->state&MIDDLEBUTTONMASK)?FXGLUT_DOWN:FXGLUT_UP;
    break;
  case RIGHTBUTTON:
    button = FXGLUT_RIGHT_BUTTON;
    state = (e->state&RIGHTBUTTONMASK)?FXGLUT_DOWN:FXGLUT_UP;
    break;
  default:           button = 0; // ERROR!
  }
  mouseFunc(button, state, e->win_x, e->win_y);
  return 1;
}
long FXGLUTWindow::onMotion(FXObject*sender,FXSelector sel,void* ptr)
{
  if (!motionFunc && !passiveMotionFunc) return 1;
  FXEvent *e = (FXEvent*)ptr;
  activeMods = e->state;
  if (e->state & (LEFTBUTTONMASK|MIDDLEBUTTONMASK|RIGHTBUTTONMASK))
  {
    if (!motionFunc || !canvas->makeCurrent()) return 1;
    motionFunc(e->win_x, e->win_y);
  }
  else 
  {
    if (!passiveMotionFunc || !canvas->makeCurrent()) return 1;
    passiveMotionFunc(e->win_x, e->win_y);
  }
      
  return 1;
}

long FXGLUTWindow::onOther    (FXObject*,FXSelector,void*)
{
  return 0;
}
//////////////////////////////////////////////////////////////////////////


class FXGLUT
{
public:
  static FXGLUT& instance() {
    static FXGLUT theGLUT;
    return theGLUT;
  }
  
  // FUNCS DIRECTLY FROM GLUT
  void init(int *argc, char **argv);
  void initDisplayMode(unsigned int mode);
  void initDisplayString(const char *string);
  void initWindowPosition(int x, int y);
  void initWindowSize(int width, int height);
  void mainLoop(void);
  int  createWindow(const char *title);
  void destroyWindow(int win);
  void postRedisplay();
  void swapBuffers(void);
  int  getWindow(void);
  void setWindow(int win);
  void setWindowTitle(const char *title);
  void setCursor(int cursor);
  void displayFunc(void ( *func)(void));
  void reshapeFunc(void ( *func)(int width, int height));
  void keyboardFunc(void ( *func)(unsigned char key, int x, int y));
  void mouseFunc(void ( *func)(int button, int state, int x, int y));
  void motionFunc(void ( *func)(int x, int y));
  void passiveMotionFunc(void ( *func)(int x, int y));
  void idleFunc(void ( *func)(void));
  void specialFunc(void ( *func)(int key, int x, int y));
  void keyboardUpFunc(void ( *func)(unsigned char key, int x, int y));
  void specialUpFunc(void ( *func)(int key, int x, int y));
  int  get(GLenum type);
  int  getModifiers(void);

  // FUNCS ONLY IN FXGLUT
  FXApp *getApp() { return app; }
  FXGLUTWindow *getFXWindow(int id);
  FXGLUTWindow *getCurrentFXWindow();
  void setCurrentFXWindow( FXGLUTWindow *win );

protected:
  std::list<FXGLUTWindow*> windowList;
  int currentID;
  FXGLUTWindow *currentWindow;
  int defaultX;
  int defaultY;
  int defaultWidth;
  int defaultHeight;
  int uniqueIDs;
  int displayMode;
private:
  FXGLUT();
  FXApp *app;
};

FXGLUT::FXGLUT() :
  defaultX(100),
  defaultY(100),
  defaultWidth(800),
  defaultHeight(600),
  uniqueIDs(0)
{
  app = new FXApp("FXGLUT","FXGLUT");
}
FXGLUTWindow *FXGLUT::getFXWindow( int id )
{
  // Stupid linear search.
  std::list<FXGLUTWindow*>::iterator it = windowList.begin();
  for (; it != windowList.end(); ++it) {
    if ( (*it)->getId() == id ) {
      return *it;
    }
  }
  // Return null if window not found
  return 0;
}
FXGLUTWindow *FXGLUT::getCurrentFXWindow()
{
  return currentWindow;
}
void FXGLUT::setCurrentFXWindow( FXGLUTWindow *win )
{
  currentWindow = win;
  // make current?
}
  
void FXGLUT::init(int *argc, char **argv)
{
  app->init(*argc,argv);
}
void FXGLUT::initDisplayMode(unsigned int mode)
{
  FXGLUTWindow::defaultDisplayMode = mode;
}
void FXGLUT::initDisplayString(const char *string)
{
}
void FXGLUT::initWindowPosition(int x, int y)
{
  defaultX = x;
  defaultY = y;
}
void FXGLUT::initWindowSize(int width, int height)
{
  defaultWidth = width;
  defaultHeight = height;
}
void FXGLUT::mainLoop(void)
{
  app->create();
  app->run();
}
int  FXGLUT::createWindow(const char *title)
{
  FXGLUTWindow *win = new FXGLUTWindow(app, title, defaultX,defaultY,defaultWidth,defaultHeight);
  win->setId(++uniqueIDs);
  windowList.push_back(win);
  currentWindow = win;
  return win->getId();
}
void FXGLUT::destroyWindow(int win)
{
}
void FXGLUT::postRedisplay()
{
  currentWindow->update();
}
void FXGLUT::swapBuffers(void)
{
  currentWindow->canvas->swapBuffers();
}
int  FXGLUT::getWindow(void)
{
  return currentWindow->getId();
}
void FXGLUT::setWindow(int id)
{
  FXGLUTWindow *win = getFXWindow(id);
  if (!win) {
    fxmessage("fxglutSetWindow(): window id %d not found\n",win);
    return;
  }
  currentWindow = win;
  currentWindow->canvas->makeCurrent();
}
void FXGLUT::setWindowTitle(const char *title)
{
  currentWindow->setTitle(title);
}
void FXGLUT::setCursor(int cursor)
{
  FXDefaultCursor fxcur = DEF_ARROW_CURSOR;
  switch(cursor)
  {
  case FXGLUT_CURSOR_INHERIT:
  case FXGLUT_CURSOR_LEFT_ARROW:    fxcur = DEF_ARROW_CURSOR;      break;
  case FXGLUT_CURSOR_RIGHT_ARROW:   fxcur = DEF_RARROW_CURSOR;     break;
  case FXGLUT_CURSOR_TEXT:          fxcur = DEF_TEXT_CURSOR;       break;
  case FXGLUT_CURSOR_CROSSHAIR:     fxcur = DEF_CROSSHAIR_CURSOR;  break;
  case FXGLUT_CURSOR_UP_DOWN:       fxcur = DEF_DRAGH_CURSOR;      break;
  case FXGLUT_CURSOR_LEFT_RIGHT:    fxcur = DEF_DRAGV_CURSOR;      break;
  case FXGLUT_CURSOR_HELP:          fxcur = DEF_HELP_CURSOR;       break;
  case FXGLUT_CURSOR_TOP_SIDE:            fxcur = DEF_VSPLIT_CURSOR;   break;
  case FXGLUT_CURSOR_BOTTOM_SIDE:         fxcur = DEF_VSPLIT_CURSOR;   break;
  case FXGLUT_CURSOR_LEFT_SIDE:           fxcur = DEF_HSPLIT_CURSOR;   break;
  case FXGLUT_CURSOR_RIGHT_SIDE:          fxcur = DEF_HSPLIT_CURSOR;   break;
  case FXGLUT_CURSOR_TOP_LEFT_CORNER:     fxcur = DEF_CORNERNW_CURSOR; break;
  case FXGLUT_CURSOR_TOP_RIGHT_CORNER:    fxcur = DEF_CORNERNE_CURSOR; break;
  case FXGLUT_CURSOR_BOTTOM_RIGHT_CORNER: fxcur = DEF_CORNERSE_CURSOR; break;
  case FXGLUT_CURSOR_BOTTOM_LEFT_CORNER:  fxcur = DEF_CORNERSW_CURSOR; break;
  case FXGLUT_CURSOR_WAIT:
    currentWindow->canvas->setDefaultCursor(app->getWaitCursor());
    return;

  // THESE ARE UNIMPLEMENTED
  case FXGLUT_CURSOR_NONE:
  case FXGLUT_CURSOR_INFO:
  case FXGLUT_CURSOR_DESTROY:
  case FXGLUT_CURSOR_CYCLE:
  case FXGLUT_CURSOR_SPRAY:
  case FXGLUT_CURSOR_FULL_CROSSHAIR:
  default:
    return;
  }
  currentWindow->canvas->setDefaultCursor(app->getDefaultCursor(fxcur));
}
void FXGLUT::displayFunc(void ( *func)(void))
{
  currentWindow->displayFunc = func;
}
void FXGLUT::reshapeFunc(void ( *func)(int width, int height))
{
  currentWindow->reshapeFunc = func;
}
void FXGLUT::keyboardFunc(void ( *func)(unsigned char key, int x, int y))
{
  currentWindow->keyboardFunc = func;
}
void FXGLUT::mouseFunc(void ( *func)(int button, int state, int x, int y))
{
  currentWindow->mouseFunc = func;
}
void FXGLUT::motionFunc(void ( *func)(int x, int y))
{
  currentWindow->motionFunc = func;
}
void FXGLUT::passiveMotionFunc(void ( *func)(int x, int y))
{
  currentWindow->passiveMotionFunc = func;
}
void FXGLUT::idleFunc(void ( *func)(void))
{
  currentWindow->idleFunc = func;
  if (func) {
    app->addChore(currentWindow,currentWindow->ID_CHORE);
  }
}
void FXGLUT::specialFunc(void ( *func)(int key, int x, int y))
{
  currentWindow->specialFunc = func;
}
void FXGLUT::keyboardUpFunc(void ( *func)(unsigned char key, int x, int y))
{
  currentWindow->keyboardUpFunc = func;
}
void FXGLUT::specialUpFunc(void ( *func)(int key, int x, int y))
{
  currentWindow->specialUpFunc = func;
}
int  FXGLUT::get(GLenum type)
{
  switch(type) {
  case FXGLUT_WINDOW_X:
    return currentWindow->getX();
  case FXGLUT_WINDOW_Y:
    return currentWindow->getY();
  case FXGLUT_WINDOW_WIDTH:
    return currentWindow->getWidth();
  case FXGLUT_WINDOW_HEIGHT:
    return currentWindow->getHeight();
  case FXGLUT_WINDOW_BUFFER_SIZE:
    //!! NOT RIGHT FOR INDEX MODES!!
    return 
      currentWindow->visual->getActualRedSize() +
      currentWindow->visual->getActualGreenSize() +
      currentWindow->visual->getActualBlueSize() +
      currentWindow->visual->getActualAlphaSize();
  case FXGLUT_WINDOW_STENCIL_SIZE:
    return currentWindow->visual->getActualStencilSize();
  case FXGLUT_WINDOW_DEPTH_SIZE:
    return currentWindow->visual->getActualDepthSize();
  case FXGLUT_WINDOW_RED_SIZE:
    return currentWindow->visual->getActualRedSize();
  case FXGLUT_WINDOW_GREEN_SIZE:
    return currentWindow->visual->getActualGreenSize();
  case FXGLUT_WINDOW_BLUE_SIZE:
    return currentWindow->visual->getActualBlueSize();
  case FXGLUT_WINDOW_ALPHA_SIZE:
    return currentWindow->visual->getActualAlphaSize();
  case FXGLUT_WINDOW_ACCUM_RED_SIZE:
    return currentWindow->visual->getActualAccumRedSize();
  case FXGLUT_WINDOW_ACCUM_GREEN_SIZE:
    return currentWindow->visual->getActualAccumGreenSize();
  case FXGLUT_WINDOW_ACCUM_BLUE_SIZE:
    return currentWindow->visual->getActualAccumBlueSize();
  case FXGLUT_WINDOW_ACCUM_ALPHA_SIZE:
    return currentWindow->visual->getActualAccumAlphaSize();
  case FXGLUT_WINDOW_DOUBLEBUFFER:
    return currentWindow->visual->isDoubleBuffer();
  case FXGLUT_WINDOW_RGBA:
    return currentWindow->visual->getType()!=VISUALTYPE_TRUE;
  case FXGLUT_WINDOW_PARENT:
    return 0;  // all top-level windows in fxglut
  case FXGLUT_WINDOW_NUM_CHILDREN:
    return 0;  // all top-level windows in fxglut
  case FXGLUT_WINDOW_COLORMAP_SIZE:
    if (currentWindow->visual->getType()==VISUALTYPE_TRUE) return 0;
    else return currentWindow->visual->getNumColors();
  case FXGLUT_WINDOW_NUM_SAMPLES:
    return 1; // no GLUT_MULTISAMPLE supported
  case FXGLUT_WINDOW_STEREO:
    return currentWindow->visual->isStereo();
  case FXGLUT_SCREEN_WIDTH:
    return app->getRootWindow()->getWidth();
  case FXGLUT_SCREEN_HEIGHT:
    return app->getRootWindow()->getHeight();
  case FXGLUT_INIT_WINDOW_X:
    return defaultX;
  case FXGLUT_INIT_WINDOW_Y:
    return defaultY;
  case FXGLUT_INIT_WINDOW_WIDTH:
    return defaultWidth;
  case FXGLUT_INIT_WINDOW_HEIGHT:
    return defaultHeight;
  ////  THE REST ARE NOT SUPPORTED ////
  case FXGLUT_INIT_DISPLAY_MODE:
  case FXGLUT_DISPLAY_MODE_POSSIBLE:
  case FXGLUT_WINDOW_CURSOR:
  case FXGLUT_SCREEN_WIDTH_MM:
  case FXGLUT_SCREEN_HEIGHT_MM:
  case FXGLUT_MENU_NUM_ITEMS:
  case FXGLUT_ELAPSED_TIME:
  case FXGLUT_WINDOW_FORMAT_ID:
  default:
    fxerror("fxglutGet(): Unknown/unsupported enumerant, %d\n",type);
  }
  return 0;
}
int  FXGLUT::getModifiers(void)
{
  int mods = currentWindow->activeMods;
  int glutmods = 0;

  if (mods & SHIFTMASK)      { glutmods |= FXGLUT_ACTIVE_SHIFT ; }
  //if (mods & CAPSLOCKMASK) { glutmods |= FXGLUT_ ; }
  if (mods & CONTROLMASK)    { glutmods |= FXGLUT_ACTIVE_CTRL ; }
  if (mods & ALTMASK)        { glutmods |= FXGLUT_ACTIVE_ALT ; }
  if (mods & METAMASK)       { glutmods |= FXGLUT_ACTIVE_ALT ; }
  //if (mods & NUMLOCKMASK)  { glutmods |= FXGLUT_ ; }
  //if (mods & SCROLLLOCKMASK)   { glutmods |= FXGLUT_ ; }
  //if (mods & LEFTBUTTONMASK)   { glutmods |= FXGLUT_ ; }
  //if (mods & MIDDLEBUTTONMASK) { glutmods |= FXGLUT_ ; }
  //if (mods & RIGHTBUTTONMASK)  { glutmods |= FXGLUT_ ; }

  return glutmods;
}
  




void fxglutInit(int *argc, char **argv)
{
  FXGLUT::instance().init(argc,argv);
}
void fxglutInitDisplayMode(unsigned int mode)
{
  FXGLUT::instance().initDisplayMode(mode);
}
void fxglutInitDisplayString(const char *string)
{
  FXGLUT::instance().initDisplayString(string);
}
void fxglutInitWindowPosition(int x, int y)
{
  FXGLUT::instance().initWindowPosition(x, y);
}
void fxglutInitWindowSize(int width, int height)
{
  FXGLUT::instance().initWindowSize(width, height);
}
void fxglutMainLoop(void)
{
  FXGLUT::instance().mainLoop();
}
int  fxglutCreateWindow(const char *title)
{
  return FXGLUT::instance().createWindow(title);
}
void fxglutDestroyWindow(int win)
{
  FXGLUT::instance().destroyWindow(win);
}
void fxglutPostRedisplay()
{
  FXGLUT::instance().postRedisplay();
}
void fxglutSwapBuffers(void)
{
  FXGLUT::instance().swapBuffers();
}
int  fxglutGetWindow(void)
{
  return FXGLUT::instance().getWindow();
}
void fxglutSetWindow(int win)
{
  FXGLUT::instance().setWindow(win);
}
void fxglutSetCursor(int cursor)
{
  FXGLUT::instance().setCursor(cursor);
}
void fxglutSetWindowTitle(const char *title)
{
  FXGLUT::instance().setWindowTitle(title);
}
void fxglutDisplayFunc(void ( *func)(void))
{
  FXGLUT::instance().displayFunc(func);
}
void fxglutReshapeFunc(void ( *func)(int width, int height))
{
  FXGLUT::instance().reshapeFunc(func);
}
void fxglutKeyboardFunc(void ( *func)(unsigned char key, int x, int y))
{
  FXGLUT::instance().keyboardFunc(func);
}
void fxglutMouseFunc(void ( *func)(int button, int state, int x, int y))
{
  FXGLUT::instance().mouseFunc(func);
}
void fxglutMotionFunc(void ( *func)(int x, int y))
{
  FXGLUT::instance().motionFunc(func);
}
void fxglutPassiveMotionFunc(void ( *func)(int x, int y))
{
  FXGLUT::instance().passiveMotionFunc(func);
}
void fxglutIdleFunc(void ( *func)(void))
{
  FXGLUT::instance().idleFunc(func);
}
void fxglutSpecialFunc(void ( *func)(int key, int x, int y))
{
  FXGLUT::instance().specialFunc(func);
}
void fxglutKeyboardUpFunc(void ( *func)(unsigned char key, int x, int y))
{
  FXGLUT::instance().keyboardUpFunc(func);
}
void fxglutSpecialUpFunc(void ( *func)(int key, int x, int y))
{
  FXGLUT::instance().specialUpFunc(func);
}
int  fxglutGet(GLenum type)
{
  return FXGLUT::instance().get(type);
}
int  fxglutGetModifiers(void)
{
  return FXGLUT::instance().getModifiers();
}

///////////////////////////////////////////////////////////////////////////
// Functions for FX integration
///////////////////////////////////////////////////////////////////////////

FXApp *fxglutGetApp()
{
  return FXGLUT::instance().getApp();
}
FXWindow *fxglutGetFXWindow(int id)
{ 
  return FXGLUT::instance().getFXWindow(id);
}
