/*---------------------------------------------------------------------------
     File name    : fxglut.h
     Author       : William Baxter
     Created      : 8/10/2003
  
     Description  : Routines to ease GLUT->FOX migration
                    This implements a good chunk of the GLUT API in terms of
                    FOX.  Currently this lacks the more exotic GLUT features
                    like overlays, subwindows, menus, geometric primitives,
                    and fonts.  For the latter you continue to use GLUT. Those
                    features are independent of GLUT's job as windowing system.

                    Because it is possible to include both glut.h and fxglut.h,
                    all the functions here are prefixed with fx to avoid 
                    naming clashes.

                    If you've got a pretty ordinary simple glut app with a few
                    callbacks set up, it should be pretty easy to convert over
                    to fxglut.  Just replace the word "glut" everywhere with
                    "fxglut" (and "GLUT" with "FXGLUT").  Add fox and fxglut
                    to libs to link with, and it should just work.

                    When you're ready to add FOX-specific stuff to your app,
                    you can get pointers to the FOX stuff used by the FXGLUT
                    implementation under the covers with these functions:
                        FXApp* fxglutGetApp();
                        FXWindow* fxglutGetFXWindow(int id);
  --------------------------------------------------------------------------*/
#ifndef _FXGLUT_H
#define _FXGLUT_H

#include <fx.h>
#include <fx3d.h>
#include <GL/glu.h>

/* Display mode bit masks. */
#define FXGLUT_RGB			0
#define FXGLUT_RGBA			FXGLUT_RGB
#define FXGLUT_INDEX			1
#define FXGLUT_SINGLE			0
#define FXGLUT_DOUBLE			2
#define FXGLUT_ACCUM			4
#define FXGLUT_ALPHA			8
#define FXGLUT_DEPTH			16
#define FXGLUT_STENCIL			32
#define FXGLUT_MULTISAMPLE		128
#define FXGLUT_STEREO			256
#define FXGLUT_LUMINANCE			512

/* Mouse buttons. */
#define FXGLUT_LEFT_BUTTON		0
#define FXGLUT_MIDDLE_BUTTON		1
#define FXGLUT_RIGHT_BUTTON		2

/* Mouse button  state. */
#define FXGLUT_DOWN			0
#define FXGLUT_UP				1

/* function keys */
#define FXGLUT_KEY_F1			1
#define FXGLUT_KEY_F2			2
#define FXGLUT_KEY_F3			3
#define FXGLUT_KEY_F4			4
#define FXGLUT_KEY_F5			5
#define FXGLUT_KEY_F6			6
#define FXGLUT_KEY_F7			7
#define FXGLUT_KEY_F8			8
#define FXGLUT_KEY_F9			9
#define FXGLUT_KEY_F10			10
#define FXGLUT_KEY_F11			11
#define FXGLUT_KEY_F12			12
/* directional keys */
#define FXGLUT_KEY_LEFT			100
#define FXGLUT_KEY_UP			101
#define FXGLUT_KEY_RIGHT			102
#define FXGLUT_KEY_DOWN			103
#define FXGLUT_KEY_PAGE_UP		104
#define FXGLUT_KEY_PAGE_DOWN		105
#define FXGLUT_KEY_HOME			106
#define FXGLUT_KEY_END			107
#define FXGLUT_KEY_INSERT			108

/* Entry/exit  state. */
#define FXGLUT_LEFT			0
#define FXGLUT_ENTERED			1

/* Menu usage  state. */
#define FXGLUT_MENU_NOT_IN_USE		0
#define FXGLUT_MENU_IN_USE		1

/* Visibility  state. */
#define FXGLUT_NOT_VISIBLE		0
#define FXGLUT_VISIBLE			1

/* Window status  state. */
#define FXGLUT_HIDDEN			0
#define FXGLUT_FULLY_RETAINED		1
#define FXGLUT_PARTIALLY_RETAINED		2
#define FXGLUT_FULLY_COVERED		3

/* Color index component selection values. */
#define FXGLUT_RED			0
#define FXGLUT_GREEN			1
#define FXGLUT_BLUE			2

/* glutGet parameters. */
#define FXGLUT_WINDOW_X			((GLenum) 100)
#define FXGLUT_WINDOW_Y			((GLenum) 101)
#define FXGLUT_WINDOW_WIDTH		((GLenum) 102)
#define FXGLUT_WINDOW_HEIGHT		((GLenum) 103)
#define FXGLUT_WINDOW_BUFFER_SIZE		((GLenum) 104)
#define FXGLUT_WINDOW_STENCIL_SIZE	((GLenum) 105)
#define FXGLUT_WINDOW_DEPTH_SIZE		((GLenum) 106)
#define FXGLUT_WINDOW_RED_SIZE		((GLenum) 107)
#define FXGLUT_WINDOW_GREEN_SIZE		((GLenum) 108)
#define FXGLUT_WINDOW_BLUE_SIZE		((GLenum) 109)
#define FXGLUT_WINDOW_ALPHA_SIZE		((GLenum) 110)
#define FXGLUT_WINDOW_ACCUM_RED_SIZE	((GLenum) 111)
#define FXGLUT_WINDOW_ACCUM_GREEN_SIZE	((GLenum) 112)
#define FXGLUT_WINDOW_ACCUM_BLUE_SIZE	((GLenum) 113)
#define FXGLUT_WINDOW_ACCUM_ALPHA_SIZE	((GLenum) 114)
#define FXGLUT_WINDOW_DOUBLEBUFFER	((GLenum) 115)
#define FXGLUT_WINDOW_RGBA		((GLenum) 116)
#define FXGLUT_WINDOW_PARENT		((GLenum) 117)
#define FXGLUT_WINDOW_NUM_CHILDREN	((GLenum) 118)
#define FXGLUT_WINDOW_COLORMAP_SIZE	((GLenum) 119)
#define FXGLUT_WINDOW_NUM_SAMPLES		((GLenum) 120)
#define FXGLUT_WINDOW_STEREO		((GLenum) 121)
#define FXGLUT_WINDOW_CURSOR		((GLenum) 122)
#define FXGLUT_SCREEN_WIDTH		((GLenum) 200)
#define FXGLUT_SCREEN_HEIGHT		((GLenum) 201)
#define FXGLUT_SCREEN_WIDTH_MM		((GLenum) 202)
#define FXGLUT_SCREEN_HEIGHT_MM		((GLenum) 203)
#define FXGLUT_MENU_NUM_ITEMS		((GLenum) 300)
#define FXGLUT_DISPLAY_MODE_POSSIBLE	((GLenum) 400)
#define FXGLUT_INIT_WINDOW_X		((GLenum) 500)
#define FXGLUT_INIT_WINDOW_Y		((GLenum) 501)
#define FXGLUT_INIT_WINDOW_WIDTH		((GLenum) 502)
#define FXGLUT_INIT_WINDOW_HEIGHT		((GLenum) 503)
#define FXGLUT_INIT_DISPLAY_MODE		((GLenum) 504)
#define FXGLUT_ELAPSED_TIME		((GLenum) 700)
#define FXGLUT_WINDOW_FORMAT_ID		((GLenum) 123)

/* glutGetModifiers return mask. */
#define FXGLUT_ACTIVE_SHIFT               1
#define FXGLUT_ACTIVE_CTRL                2
#define FXGLUT_ACTIVE_ALT                 4


#define FXGLUT_CURSOR_RIGHT_ARROW		0
#define FXGLUT_CURSOR_LEFT_ARROW		1
/* Symbolic cursor shapes. */
#define FXGLUT_CURSOR_INFO		2
#define FXGLUT_CURSOR_DESTROY		3
#define FXGLUT_CURSOR_HELP		4
#define FXGLUT_CURSOR_CYCLE		5
#define FXGLUT_CURSOR_SPRAY		6
#define FXGLUT_CURSOR_WAIT		7
#define FXGLUT_CURSOR_TEXT		8
#define FXGLUT_CURSOR_CROSSHAIR		9
/* Directional cursors. */
#define FXGLUT_CURSOR_UP_DOWN		10
#define FXGLUT_CURSOR_LEFT_RIGHT		11
/* Sizing cursors. */
#define FXGLUT_CURSOR_TOP_SIDE		12
#define FXGLUT_CURSOR_BOTTOM_SIDE		13
#define FXGLUT_CURSOR_LEFT_SIDE		14
#define FXGLUT_CURSOR_RIGHT_SIDE		15
#define FXGLUT_CURSOR_TOP_LEFT_CORNER	16
#define FXGLUT_CURSOR_TOP_RIGHT_CORNER	17
#define FXGLUT_CURSOR_BOTTOM_RIGHT_CORNER	18
#define FXGLUT_CURSOR_BOTTOM_LEFT_CORNER	19
/* Inherit from parent window. */
#define FXGLUT_CURSOR_INHERIT		100
/* Blank cursor. */
#define FXGLUT_CURSOR_NONE		101
/* Fullscreen crosshair (if available). */
#define FXGLUT_CURSOR_FULL_CROSSHAIR	102

void fxglutInit(int *argcp, char **argv);
void fxglutInitDisplayMode(unsigned int mode);
void fxglutInitDisplayString(const char *string);
void fxglutInitWindowPosition(int x, int y);
void fxglutInitWindowSize(int width, int height);
void fxglutMainLoop(void);
int  fxglutCreateWindow(const char *title);
//int  fxglutCreateSubWindow(int win, int x, int y, int width, int height);
void fxglutDestroyWindow(int win);
void fxglutPostRedisplay(void);
//void fxglutPostWindowRedisplay(int win);
void fxglutSwapBuffers(void);
int  fxglutGetWindow(void);
void fxglutSetWindow(int win);
void fxglutSetWindowTitle(const char *title);
//void fxglutSetIconTitle(const char *title);
//void fxglutPositionWindow(int x, int y);
//void fxglutReshapeWindow(int width, int height);
//void fxglutPopWindow(void);
//void fxglutPushWindow(void);
//void fxglutIconifyWindow(void);
//void fxglutShowWindow(void);
//void fxglutHideWindow(void);
//void fxglutFullScreen(void);
void fxglutSetCursor(int cursor);
//void fxglutWarpPointer(int x, int y);
//void fxglutEstablishOverlay(void);
//void fxglutRemoveOverlay(void);
//void fxglutUseLayer(GLenum layer);
//void fxglutPostOverlayRedisplay(void);
//void fxglutPostWindowOverlayRedisplay(int win);
//void fxglutShowOverlay(void);
//void fxglutHideOverlay(void);
//int  fxglutCreateMenu(void (GLUTCALLBACK *func)(int));
//void fxglutDestroyMenu(int menu);
//int  fxglutGetMenu(void);
//void fxglutSetMenu(int menu);
//void fxglutAddMenuEntry(const char *label, int value);
//void fxglutAddSubMenu(const char *label, int submenu);
//void fxglutChangeToMenuEntry(int item, const char *label, int value);
//void fxglutChangeToSubMenu(int item, const char *label, int submenu);
//void fxglutRemoveMenuItem(int item);
//void fxglutAttachMenu(int button);
//void fxglutDetachMenu(int button);
void fxglutDisplayFunc(void ( *func)(void));
void fxglutReshapeFunc(void ( *func)(int width, int height));
void fxglutKeyboardFunc(void ( *func)(unsigned char key, int x, int y));
void fxglutMouseFunc(void ( *func)(int button, int state, int x, int y));
void fxglutMotionFunc(void ( *func)(int x, int y));
void fxglutPassiveMotionFunc(void ( *func)(int x, int y));
//void fxglutEntryFunc(void ( *func)(int state));
//void fxglutVisibilityFunc(void ( *func)(int state));
void fxglutIdleFunc(void ( *func)(void));
//void fxglutTimerFunc(unsigned int millis, void ( *func)(int value), int value);
//void fxglutMenuStateFunc(void ( *func)(int state));
void fxglutSpecialFunc(void ( *func)(int key, int x, int y));
//void fxglutSpaceballMotionFunc(void ( *func)(int x, int y, int z));
//void fxglutSpaceballRotateFunc(void ( *func)(int x, int y, int z));
//void fxglutSpaceballButtonFunc(void ( *func)(int button, int state));
//void fxglutButtonBoxFunc(void ( *func)(int button, int state));
//void fxglutDialsFunc(void ( *func)(int dial, int value));
//void fxglutTabletMotionFunc(void ( *func)(int x, int y));
//void fxglutTabletButtonFunc(void ( *func)(int button, int state, int x, int y));
//void fxglutMenuStatusFunc(void ( *func)(int status, int x, int y));
//void fxglutOverlayDisplayFunc(void ( *func)(void));
//void fxglutWindowStatusFunc(void ( *func)(int state));
void fxglutKeyboardUpFunc(void ( *func)(unsigned char key, int x, int y));
void fxglutSpecialUpFunc(void ( *func)(int key, int x, int y));
//void fxglutJoystickFunc(void ( *func)(unsigned int buttonMask, int x, int y, int z), int pollInterval);
//void fxglutSetColor(int, GLfloat red, GLfloat green, GLfloat blue);
//GLfloat fxglutGetColor(int ndx, int component);
//void fxglutCopyColormap(int win);
int  fxglutGet(GLenum type);
//int  fxglutDeviceGet(GLenum type);
//int  fxglutExtensionSupported(const char *name);
int  fxglutGetModifiers(void);
//int  fxglutLayerGet(GLenum type);
//void fxglutBitmapCharacter(void *font, int character);
//int  fxglutBitmapWidth(void *font, int character);
//void fxglutStrokeCharacter(void *font, int character);
//int  fxglutStrokeWidth(void *font, int character);
//int  fxglutBitmapLength(void *font, const unsigned char *string);
//int  fxglutStrokeLength(void *font, const unsigned char *string);
//void fxglutWireSphere(GLdouble radius, GLint slices, GLint stacks);
//void fxglutSolidSphere(GLdouble radius, GLint slices, GLint stacks);
//void fxglutWireCone(GLdouble base, GLdouble height, GLint slices, GLint stacks);
//void fxglutSolidCone(GLdouble base, GLdouble height, GLint slices, GLint stacks);
//void fxglutWireCube(GLdouble size);
//void fxglutSolidCube(GLdouble size);
//void fxglutWireTorus(GLdouble innerRadius, GLdouble outerRadius, GLint sides, GLint rings);
//void fxglutSolidTorus(GLdouble innerRadius, GLdouble outerRadius, GLint sides, GLint rings);
//void fxglutWireDodecahedron(void);
//void fxglutSolidDodecahedron(void);
//void fxglutWireTeapot(GLdouble size);
//void fxglutSolidTeapot(GLdouble size);
//void fxglutWireOctahedron(void);
//void fxglutSolidOctahedron(void);
//void fxglutWireTetrahedron(void);
//void fxglutSolidTetrahedron(void);
//void fxglutWireIcosahedron(void);
//void fxglutSolidIcosahedron(void);
//int  fxglutVideoResizeGet(GLenum param);
//void fxglutSetupVideoResizing(void);
//void fxglutStopVideoResizing(void);
//void fxglutVideoResize(int x, int y, int width, int height);
//void fxglutVideoPan(int x, int y, int width, int height);
//void fxglutReportErrors(void);
//void fxglutIgnoreKeyRepeat(int ignore);
//void fxglutSetKeyRepeat(int repeatMode);
//void fxglutForceJoystickFunc(void);
//void fxglutGameModeString(const char *string);
//int  fxglutEnterGameMode(void);
//void fxglutLeaveGameMode(void);
//int  fxglutGameModeGet(GLenum mode);


///////////////////////////////////////////////////////////////////////////
// Functions for FX integration
///////////////////////////////////////////////////////////////////////////

FXApp *fxglutGetApp();
FXWindow *fxglutGetFXWindow(int id);

#endif // _FXGLUT_H
