
/*
 * mondrian.c
 * 
 * Draw a bunch of colored rectangles in the style of the painter Mondrain.
 *
 *   Left button: randomizes the pattern
 *   Middle button: toggles between filled and outlined rectangles.
 *
 * Pat Hanrahan 1/97
 */
#include "fxglut.h"
#include <stdlib.h>

GLint Width = 512, Height = 512;
int   Times = 100;
int   FillFlag  = 1;
long  Seed = 0;

#ifdef _WIN32
inline double drand48()
{
  return  rand()/double(RAND_MAX);
}
inline void srand48(long s)
{
  srand(s);
}
inline long mrand48()
{
  return rand();
}
#endif


void DrawRect( float x1, float y1, float x2, float y2, int FillFlag )
{
  glBegin(FillFlag ? GL_QUADS : GL_LINE_LOOP);
  glVertex2f(x1, y1);
  glVertex2f(x2, y1);
  glVertex2f(x2, y2);
  glVertex2f(x1, y2);
  glEnd();
}

void Display(void)
{
  int i;
  float x1, y1, x2, y2;
  float r, g, b;
  
  srand48(Seed);
  
  glClearColor(0, 0, 0, 1);
  glClear(GL_COLOR_BUFFER_BIT);
  
  for( i = 0; i < Times; i++ ) {
    r = drand48();
    g = drand48();
    b = drand48();
    glColor3f( r, g, b );
    
    x1 = drand48() * Width;
    y1 = drand48() * Height;
    x2 = drand48() * Width;
    y2 = drand48() * Height;
    DrawRect( x1, y1, x2, y2, FillFlag );
  }
  glFinish();
  fxglutSwapBuffers();
}

void Motion(GLint x, GLint y)
{
  int db;
  glGetIntegerv(GL_DRAW_BUFFER, &db);
  glDrawBuffer(GL_FRONT);
  glColor3f(1,1,1);
  glBegin(GL_POINTS);
  glVertex2f(x,Height-y);
  glEnd();
  glDrawBuffer(db);
}

void Reshape(GLint w, GLint h)
{
  Width = w;
  Height = h;
  
  glViewport(0, 0, w, h);
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, w, 0, h, -1.0, 1.0);
  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

void Mouse(int button, int state, int x, int y)
{
  if( state == FXGLUT_DOWN ) {
    switch( button ) {
    case FXGLUT_LEFT_BUTTON:
      Seed = mrand48();
      break;
    case FXGLUT_MIDDLE_BUTTON:
      FillFlag = !FillFlag;
      break;
    }
    fxglutPostRedisplay();
  }
}

void Idle() {
  Seed = mrand48();
  fxglutPostRedisplay();
}

void Keyboard( unsigned char key, int x, int y )
{
#define ESCAPE '\033'

    if( key == ESCAPE )
        exit(0);
    if( key == 't' ) {
      fxglutSetWindowTitle("A New Title!");
    }
    if( key == 'c' ) {
      fxglutSetCursor(FXGLUT_CURSOR_WAIT);
    }
    if( key == 'C' ) {
      fxglutSetCursor(FXGLUT_CURSOR_INHERIT);
    }
    if (key == 13 ) {
      printf("\n");
    }
    if (key == 'i') {
      fxglutIdleFunc(Idle);
    }
    if (key == 'I') {
      fxglutIdleFunc(0);
    }
    if (key == 'p') {
      fxglutPassiveMotionFunc(Motion);
    }
    if (key == 'P') {
      fxglutPassiveMotionFunc(0);
    }
    printf("%c", key);
}
void Special( int key, int x, int y )
{
  switch(key)
  {
  case FXGLUT_KEY_F1: printf("F1!\n"); break;
  case FXGLUT_KEY_F2: printf("F2!\n"); break;
  case FXGLUT_KEY_F3: printf("F3!\n"); break;
  case FXGLUT_KEY_F4: printf("F4!\n"); break;
  case FXGLUT_KEY_F5: printf("F5!\n"); break;
  case FXGLUT_KEY_F12: printf("F12!\n"); break;
  case FXGLUT_KEY_INSERT: printf("INSERT!\n"); break;
  case FXGLUT_KEY_UP: printf("UP!\n"); break;
  case FXGLUT_KEY_DOWN: printf("DOWN!\n"); break;
  case FXGLUT_KEY_HOME: printf("HOME!\n"); break;
  case FXGLUT_KEY_END: printf("END!\n"); break;

  }
}

int main(int argc, char *argv[])
{
  fxglutInit(&argc, argv);
  fxglutInitDisplayMode(FXGLUT_RGB|FXGLUT_DOUBLE);

  fxglutInitWindowSize(512,512);
  fxglutInitWindowPosition(100,100);

  int id = fxglutCreateWindow("Mondrian Example!");

  fxglutDisplayFunc(Display);
  fxglutReshapeFunc(Reshape);
  fxglutKeyboardFunc(Keyboard);
  fxglutSpecialFunc(Special);
  fxglutMouseFunc(Mouse);
  fxglutMotionFunc(Motion);

  fxglutMainLoop();
  
  return 0;
}
